# -*- coding: utf-8 -*-
# Copyright 2009-2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

from __future__ import absolute_import

from libtaskotron.koji_utils import KojiClient
from libtaskotron.logger import log
from libtaskotron.directives import BaseDirective
from libtaskotron.exceptions import TaskotronDirectiveError
from libtaskotron import rpm_utils

directive_class = 'KojiDirective'

class KojiDirective(BaseDirective):
    """The koji directive interfaces with Koji to facilitate various koji
    actions.

    Format::

        koji: target_dir=<target_dir> command=[download, download_tag] <command args>

    Valid Commands:

    download
      Downloads the koji build specified as ``koji_build``

      Format::

        koji: command=download koji_build=<N(E)VR to download> arch=<desired arch>

    download_tag
      Downloads rpms tagged by ``koji_tag``

      Format::

        koji: command=download_tag koji_tag=<koji tag to download> arch=<desired arch>


    Additional supported ``arch`` argument value: ``src``

    Note about the ``arch`` argument: if not requested, ``noarch`` is added to the list
    of arches (unless ``src`` is the only one requested)
    """

    def __init__(self, koji_session=None):
        super(KojiDirective, self).__init__()
        if koji_session is None:
            self.koji = KojiClient(koji_session)
        else:
            self.koji = koji_session

    def process(self, input_data, env_data):
        valid_actions = ['download', 'download_tag']

        output_data = {}

        action = input_data['action']

        if not action in valid_actions:
            raise TaskotronDirectiveError('%s is not a valid command for koji '
                                          'helper' % action)

        if 'arch' not in input_data:
            detected_args = ', '.join(input_data.keys())
            raise TaskotronDirectiveError(
                "The koji directive requires 'arch' as an argument. Detected"
                "arguments: %s" % detected_args)

        if not 'target_dir' in input_data:
            self.target_dir = env_data['workdir']
        else:
            self.target_dir = input_data['target_dir']

        self.arches = input_data['arch']

        if 'noarch' not in self.arches and not self.arches == ['src']:
            self.arches.append('noarch')

        if action == 'download':
            if 'koji_build' not in input_data:
                detected_args = ', '.join(input_data.keys())
                raise TaskotronDirectiveError(
                    "The koji directive requires 'koji_build' for the 'download'"
                    "action. Detected arguments: %s" % detected_args)

            nvr = rpm_utils.rpmformat(input_data['koji_build'], 'nvr')

            log.info("Getting RPMs for koji build %s (%s) and downloading to %s",
                     nvr, str(self.arches), self.target_dir)
            output_data['downloaded_rpms'] = self.koji.get_nvr_rpms(nvr,
                                                                self.target_dir,
                                                                arches=self.arches,
                                                                src=('src' in self.arches))
        elif action == 'download_tag':
            if 'koji_tag' not in input_data:
                detected_args = ', '.join(input_data.keys())
                raise TaskotronDirectiveError(
                    "The koji directive requires 'koji_tag' for the 'download_tag'"
                    "action. Detected arguments: %s" % detected_args)

            koji_tag = input_data['koji_tag']

            log.info("Getting koji builds for koji tag '%s', arches '%s' and "
                     "downloading to %s", koji_tag, str(self.arches), self.target_dir)
            output_data['downloaded_rpms'] = self.koji.get_tagged_rpms(koji_tag,
                                                                self.target_dir,
                                                                self.arches)

        return output_data
