# -*- coding: utf-8 -*-
# Copyright 2009-2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

from __future__ import absolute_import
from libtaskotron.directives import BaseDirective

from libtaskotron import check
from libtaskotron import config
from libtaskotron import buildbot_utils

from libtaskotron.exceptions import TaskotronDirectiveError, TaskotronValueError
from libtaskotron.logger import log

import resultsdb_api

directive_class = 'ResultsdbDirective'

class ResultsdbDirective(BaseDirective):
    """The resultsdb directive interfaces with ResultsDB to store results.

    When reporting is disabled, the directive checks whether the input is a valid
    TAP and lists out the details that would have been reported.

    Format::

        resultsdb: results=<TAP13>
    """

    def __init__(self, resultsdb = None):
        self.resultsdb = resultsdb

        conf = config.get_config()
        self.masterurl = conf.taskotron_master
        self.task_stepname = conf.buildbot_task_step

        if self.resultsdb is None:
            self.resultsdb = resultsdb_api.ResultsDBapi(conf.resultsdb_server)

    def ensure_testcase_exists(self, name):
        """ Make sure that the testcase exists in resultsdb, otherwise create
        the testcase using a dummy url as a reference

        :param str name: name of the testcase to check for
        """

        try:
            testcase = self.resultsdb.get_testcase(name)
            return
        except resultsdb_api.ResultsDBapiException, e:
            if e.message != 'Testcase not found':
                raise e

        # since the testcase doesn't exist, create it with a dummy value for url
        # it can be updated later when it's not blocking results reporting
        dummyurl = 'http://faketestcasesRus.com/%s' % name
        self.resultsdb.create_testcase(name, dummyurl)

    def create_resultsdb_job(self, name, refurl=None):
        """ Create a job in resultsdb for reporting results against

        :param str name: name of job to report against
        :returns: dict of rendered json results
        """

        # this is completly hacky for now since the runner doesn't have a
        # good way to get the actual refurl. This will be addressed in T122 and
        # T145

        url = 'http://not-a-real-domain.com/noworky'
        if refurl is not None:
            url = refurl

        jobdata = self.resultsdb.create_job(url, status='SCHEDULED', name=name)
        self.resultsdb.update_job(id=jobdata['id'], status='RUNNING')

        return jobdata

    def complete_resultsdb_job(self, jobid):
        """ Change the resultsdb job to a status of ``COMPLETED``, indicating
        that the reporting is complete.

        :param int jobid: The resultsdb jobid to be modified
        :returns: json representation of returned data from the resultsdb instance
        """
        return self.resultsdb.update_job(jobid, status='COMPLETED')

    def process(self, input_data, env_data):
        #TODO: Replace with proper location and adjust exc. text
        # we're creating the jobid in the directive for now, so adjust the check

        if 'checkname' not in env_data:
            raise TaskotronDirectiveError("The resultsdb directive requires "\
                    "resultsdb_job_id and checkname.")

        # checking if reporting is enabled is done after importing tap which
        # serves as validation of input results
        try:
            check_details = check.import_TAP(input_data['results'])
        except TaskotronValueError as e:
            raise TaskotronDirectiveError("Failed to load 'results': %s"  % e.message)

        conf = config.get_config()
        if not (conf.reporting_enabled and conf.report_to_resultsdb):
            log.info("TAP is OK.")
            log.info("Reporting to ResultsDB is disabled.")
            log.info("Once enabled, the following would be reported:")
            log.info('\n'.join([str(detail) for detail in check_details]))
            return

        # for now, we're creating the resultsdb job at reporting time
        # the job needs to be 'RUNNING' in order to append any results
        job_url, log_url = buildbot_utils.get_urls(env_data['jobid'],
                                                   self.masterurl,
                                                   self.task_stepname)
        job_data = self.create_resultsdb_job(env_data['checkname'],
                                             refurl=job_url)

        self.ensure_testcase_exists(env_data['checkname'])

        output = []

        for detail in check_details:
            try:
                result = self.resultsdb.create_result(
                        job_id = job_data['id'],
                        testcase_name = env_data['checkname'],
                        outcome = detail.outcome,
                        summary = detail.summary or None,
                        log_url = log_url,
                        item = detail.item,
                        type = detail.report_type,
                        **detail.keyvals)
                output.append(result)

            except resultsdb_api.ResultsDBapiException, e:
                log.error(e)
                log.error("Failed to store ResulsDB: `%s` `%s` `%s`",
                            detail.item, env_data['checkname'], detail.outcome)

        # move job to completed
        self.complete_resultsdb_job(job_data['id'])

        return output

