# -*- coding: utf-8 -*-
# Copyright 2009-2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

from __future__ import absolute_import
from libtaskotron.directives import BaseDirective

from libtaskotron import config
from libtaskotron import yumrepoinfo

from libtaskotron.exceptions import TaskotronDirectiveError
from libtaskotron.logger import log

directive_class = 'YumrepoinfoDirective'

class YumrepoinfoDirective(BaseDirective):
    """A directive for translating koji tag into a set of yum repos.
    The directive returns a dictionary having ``koji_tag`` as the key, and
    URL for the respective repo as a value.
    If the koji tag specified by ``koji_tag`` argument has any parent repos
    (e.g. f20-updates is a child of f20), all the repos from the ``koji_tag``
    and up are returned.
    If the ``koji_tag`` is ``-pending``, the ``-pending`` is stripped (as it is
    not a repo), and then the code works the same.
    It is also possible to assign ``koji_tag`` to ``rawhide``.

    Format::

        yumrepoinfo:
        koji_tag=<tag>
        arch=<arch>
    """

    def __init__(self, repoinfo = None):
        self.repoinfo = repoinfo

    def process(self, input_data, env_data):

        if 'koji_tag' not in input_data or 'arch' not in input_data:
            raise TaskotronDirectiveError("The yumrepoinfo directive requires "\
                    "koji_tag and arch arguments.")


        arches = input_data['arch']

        processed_arches = [arch for arch in arches if arch not in ['noarch', 'all', 'src']]
        if len(processed_arches) == 0:
            raise TaskotronDirectiveError("No valid yumrepo arches supplied to "
                    "yumrepoinfo directive. Recieved %r" % arches)

        if len(processed_arches) > 1:
          raise TaskotronDirectiveError("Yumrepoinfo requires a single arch "
                    "but multiple arches were submitted: %r" % arches)

        arch = processed_arches[0]

        koji_tag = input_data['koji_tag']

        output = {}

        if self.repoinfo is None:
            self.repoinfo = yumrepoinfo.get_yumrepoinfo(arch)

        if koji_tag.endswith('-pending'):
           koji_tag = koji_tag[:-len('-pending')]

        if koji_tag == 'rawhide':
            koji_tag = self.repoinfo.repo('rawhide')['tag']

        while koji_tag:
            repo = self.repoinfo.repo_by_tag(koji_tag)
            if repo is None:
                raise TaskotronDirectiveError('Repo with tag'\
                        '%r not found.' % koji_tag)

            output[repo['name']] = repo['url']
            koji_tag = repo['parent']

        log.debug("Found %s repos for %s: %r" % (len(output), input_data['koji_tag'], output))
        return output

